/* GtkDatabox - An extension to the gtk+ library
 * Copyright (C) 1998 - 2005  Dr. Roland Bock
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include <stdlib.h>
#include <stdio.h>

#include <gtk/gtk.h>
#include <gtkdatabox.h>
#include <math.h>
/* Some versions of math.h have a PI problem... */
#ifndef PI
#define PI 3.14159265358979323846
#endif

#define POINTS 2000

static gulong continual_counter = 0;

void
gtk_databox_continual_points_draw (GtkDatabox * box, GtkDataboxData * data)
{
   gint i = 0;
   gint length = data->length;

   for (i = 0; i < length; i++)
   {
      box->points[i].x = (guint16) ((gfloat)((length
                                            - 1 
                                            - (length - 1 
                                               + continual_counter - i) 
                                            % length)
                                            - box->top_left.x)
                                    * box->factor.x);
      box->points[i].y =
	 (gint16) ((data->Y[i] - box->top_left.y) * box->factor.y);

   }
   
   if (data->size < 2)
   {
      /* More than 2^16 points will cause X IO error on most XServers
         (Hint from Paul Barton-Davis <pbd@Op.Net>) */
      for (i = 0; i < length; i += 65536)
      {
	 gdk_draw_points (box->pixmap, data->gc, box->points + i,
			  MIN (65536, length - i));
      }
   }
   else
   {
      for (i = 0; i < length; i++)
      {
	 /* Why on earth is there no gdk_draw_rectangles?? */
	 gdk_draw_rectangle (box->pixmap, data->gc, TRUE,
			     box->points[i].x - data->size / 2,
			     box->points[i].y - data->size / 2, data->size,
			     data->size);
      }
   }
}






/*----------------------------------------------------------------
 *  databox continual data
 *----------------------------------------------------------------*/

static gfloat *X = NULL;
static gfloat *Y = NULL;

static gint continual_idle = 0;
static gfloat continual_delta = 0;
static gint continual_range = 1;
static GtkWidget *continual_label = NULL;

static gboolean
continual_idle_func (GtkDatabox * box)
{
   gfloat random = 1 + (int) (20.0 * rand() / (RAND_MAX + 1.0));;
   gchar label[10];

   if (!GTK_IS_DATABOX (box))
      return FALSE;

   if (! continual_counter) Y[0] = 0;
   else 
      Y[continual_counter % POINTS] = Y [(continual_counter - 1) % POINTS] 
                                      + continual_delta;
   
   if (-- continual_range <= 0)
   {
      continual_delta = 0.1 * (-10 + (int) (21.0 * rand() / (RAND_MAX + 1.0)));
      continual_range = 1 + (int) (100.0 * rand() / (RAND_MAX + 1.0));
      if (Y[continual_counter % POINTS] + continual_delta * continual_range 
            > 100. 
          || Y[continual_counter % POINTS] + continual_delta * continual_range
            < -100.)
      {
         continual_delta = - continual_delta;
      }
      
   }
   
   gtk_databox_redraw (GTK_DATABOX (box));

   sprintf (label, "%ld", continual_counter++);
   gtk_entry_set_text (GTK_ENTRY (continual_label), label);

   return TRUE;
}

static void
create_continual (void)
{
   GtkWidget *window = NULL;
   GtkWidget *box1;
   GtkWidget *box2;
   GtkWidget *close_button;
   GtkWidget *box;
   GtkWidget *label;
   GtkWidget *separator;
   GdkColor color;
   GtkDataboxValue min, max;
   gint i;

   guint CONTINUAL_DISPLAY = 
         gtk_databox_display_type_add (
         GTK_DATABOX_DRAW_FUNC(gtk_databox_continual_points_draw),
         GTK_DATABOX_REMOVE_FUNC(NULL));

   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_widget_set_size_request (window, 800, 300);

   g_signal_connect (GTK_OBJECT (window), "destroy",
		     GTK_SIGNAL_FUNC (gtk_main_quit), NULL);

   gtk_window_set_title (GTK_WINDOW (window), "GtkDatabox: Continual Data");
   gtk_container_set_border_width (GTK_CONTAINER (window), 0);

   box1 = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), box1);

   label =
      gtk_label_new
      ("This example shows how a stream of continuously incoming data could\nbe handled. For this purpose, a new display function is defined as\npart of the example code.");
   gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 0);
   separator = gtk_hseparator_new ();
   gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, FALSE, 0);
   continual_label = gtk_entry_new ();
   gtk_entry_set_text (GTK_ENTRY (continual_label), "0");
   gtk_box_pack_start (GTK_BOX (box1), continual_label, FALSE, FALSE, 0);
   separator = gtk_hseparator_new ();
   gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, FALSE, 0);

   continual_idle = 0;
   continual_counter = 0;

   box = gtk_databox_new ();

   X = g_new0 (gfloat, POINTS);
   Y = g_new0 (gfloat, POINTS);

   for (i = 0; i < POINTS; i++)
   {
      X[i] = i;
      Y[i] = -1000.;
   }
   color.red = 65535;
   color.green = 65535;
   color.blue = 0;

   gtk_databox_data_add (GTK_DATABOX (box), POINTS,
			 X, Y, color,
			 CONTINUAL_DISPLAY, 0);

   color.red = 0;
   color.green = 0;
   color.blue = 32768;
   gtk_databox_set_background_color (GTK_DATABOX (box), color);

   min.x = 0.;
   min.y = -100.;
   max.x = POINTS;
   max.y = 100.;

   gtk_databox_rescale_with_values (GTK_DATABOX (box), min, max);

   gtk_box_pack_start (GTK_BOX (box1), box, TRUE, TRUE, 0);

   separator = gtk_hseparator_new ();
   gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);

   box2 = gtk_vbox_new (FALSE, 10);
   gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
   gtk_box_pack_end (GTK_BOX (box1), box2, FALSE, TRUE, 0);
   close_button = gtk_button_new_with_label ("close");

   g_signal_connect_swapped (GTK_OBJECT (close_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_main_quit),
			     GTK_OBJECT (box));

   gtk_box_pack_start (GTK_BOX (box2), close_button, TRUE, TRUE, 0);
   GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);
   gtk_widget_grab_default (close_button);
   continual_idle = gtk_idle_add ((GtkFunction) continual_idle_func, box);

   gtk_widget_show_all (window);
}

gint
main (gint argc, char *argv[])
{
   gtk_init (&argc, &argv);

   create_continual ();
   gtk_main ();

   return 0;
}
